#include <stdio.h>
#include "defs.h"

#define DEFAULT		0
#define ENHANCED	1
#define MANUAL		2
#define SYMMETRIC	3

ypedef struct {
	float   **image;
	short   xsize, ysize;
	int     nflag;
	float	spos, sneg;
}       FILTER;

har   *progname;
har   *filename;
ixrect *pr1, *pr2, *tmp1;
nt     scale_method;
loat   scale;
nt     offset;
har   *filtername;
hort   fcon_max_enhanced = -32768, fcon_min_enhanced = 32767;
nt     mfx, mfy, newxsize, newysize;
nt     levels;
FILTER *filter;

#ifdef STANDALONE
ain(argc, argv, envp)
#else
convolve_main(argc, argv, envp)
#endif
	int     argc;
	char  **argv;
	char  **envp;
{
	register int i, j;
	FILTER *fconvolve_loadfilter();

	scale_method = DEFAULT;
	scale = 1.;
	offset = 0;
	filtername = strsave("non-specified");
	progname = strsave(argv[0]);
	parse_profile(&argc, argv, envp);

	while ((gc = getopt(argc, argv, "sef:lm:M:")) != EOF)
		switch (gc) {
		case 's':
			scale_method = SYMMETRIC;
			break;
		case 'e':
			scale_method = ENHANCED;
			break;
		case 'f':
			filtername = strsave(optarg);
			break;
		case 'l':
			execlp("ls", "ls", "-C", FILTERS_DIR);
			exit();
			break;
		case 'm':
			scale_method = MANUAL;
			scale = atof(optarg);
			break;
		case 'M':
			scale_method = MANUAL;
			offset = (int) atof(optarg);
			break;
		case '?':
			errflag++;
			break;
		}

	if (errflag)
		error((char *) 0, "Usage: %s: [-e] [-s] [-f filter] [-l] [-m manual_scale] [-M manual_offset] [infile] [outfile]", progname);

	for (stream = 0; optind < argc; stream++, optind++)
		if (stream < 2 && strcmp(argv[optind], "-") != 0)
			if (freopen(argv[optind], mode[stream], f[stream]) == NULL)
				error("%s %s", PR_IO_ERR_INFILE, argv[optind]);

	filter = fconvolve_loadfilter(filtername);

	if ((pr1 = pr_load(stdin, NULL)) == NULL)
		error(PR_IO_ERR_RASREAD);

	if (bitrestrict)
		levels = calc_max(pr1);
	else
		levels = MAXLEVEL(pr1->pr_depth);

	if ((pr2 = mem_create(pr1->pr_size.x, pr1->pr_size.y, pr1->pr_depth)) == NULL)
		error("mem_create returned NULL");

	mfx = filter->xsize / 2;
	mfy = filter->ysize / 2;

	switch (scale_method) {
	case DEFAULT:
		fconvolve_calc_default_scale();
		fconvolve_one_pass();
		break;
	case ENHANCED:
	case SYMMETRIC:
		fconvolve_two_pass();
		break;
	case MANUAL:
		fconvolve_one_pass();
		break;
	}

	pr_dump(pr2, stdout, NULL, RT_STANDARD, 0);
}


FILTER *
convolve_loadfilter(name)
	char   *name;
{
	char    buf[BUFSIZ];
	FILE   *fp;
	char   *ep;
	FILTER *filter;
	int     xsize, ysize;
	float 	fval;
	register int i, j;
	char   *getenv();

	if ((ep = getenv("FILTERS")) == NULL)
		strcpy(buf, FILTERS_DIR);
	else
		strcpy(buf, ep);

	strcat(buf, "/");
	strcat(buf, name);

	if (!(fp = fopen(buf, "r")))
		error("Couldn't open filter %s", name);

	fscanf(fp, "%d %d", &xsize, &ysize);
	filter = (FILTER *) malloc(sizeof(FILTER));
	filter->image = (float **) dynamem(&(filter->image), sizeof(float), 2, ysize, xsize);

	filter->nflag = FALSE;
	filter->xsize = xsize;
	filter->ysize = ysize;

	for (j = 0; j < ysize; j++)
		for (i = 0; i < xsize; i++) {
			fscanf(fp, "%g", &fval);
			filter->image[j][i] = fval;
			if (fval < 0) {
				filter->nflag = TRUE;
				filter->sneg += fval;
			} else
				filter->spos += fval;
		}
	filter->sneg = fabs(filter->sneg);

	return filter;
}
nlarge_image()
{
	int i,j;
	newxsize = pr1->pr_size.x + mfx * 2;
	newysize = pr1->pr_size.y + mfy * 2;
	if ((tmp1 = mem_create(newxsize, newysize, pr1->pr_depth)) == NULL)
		error("mem_create returned NULL");
	
	for (j = 0; j<tmp1->pr_size.y; j++)
		for(i = 0; i<tmp1->pr_size.y; i++){
			if(i < mfx){
				if(j < mfy)
					pr_put(tmp1, i, j, pr_get(pr1, 0, 0));
				else if(j < newysize - mfy)
					pr_put(tmp1, i, j, pr_get(pr1, 0, j-mfy));
				else
					pr_put(tmp1, i, j, pr_get(pr1, 0, pr1->pr_size.y-1));
				}
			else if(i >= mfx && i < newxsize - mfx){
				if(j < filter->ysize/2)
					pr_put(tmp1, i, j, pr_get(pr1, i-mfx, 0));
				else if(j >= newysize-mfy)
					pr_put(tmp1, i, j, pr_get(pr1, i-mfx, pr1->pr_size.y-1));
				else
					pr_put(tmp1, i, j, pr_get(pr1, i-mfx, j-mfy));
				}
			else {
				if(j < mfy)
					pr_put(tmp1, i, j, pr_get(pr1, pr1->pr_size.x-1, 0));
				else if(j < newysize-mfy)
					pr_put(tmp1, i, j, pr_get(pr1, pr1->pr_size.x-1, j-mfy));
				else
					pr_put(tmp1, i, j, pr_get(pr1, pr1->pr_size.x-1, pr1->pr_size.y-1));
				}
			}
}

convolve_calc_default_scale()
{
	if (filter->nflag) {
		scale = 1.0 / (MAX(filter->spos, filter->sneg) * 2);
		offset = levels / 2.;
	} else {
		scale = 1.0 / (MAX(filter->spos, filter->sneg));
		offset = 0.;
	}
}
convolve_one_pass()
{
	float total;
	register int fj, fi;
	register int i, j;

	warning("scale = %g on input, offset = %d on output", scale, offset);
	
	enlarge_image();

	for (j = 0; j < tmp1->pr_size.y - filter->ysize; j++)
		for (i = 0; i < tmp1->pr_size.x - filter->xsize; i++) {
			total = 0;
			for (fj = 0; fj < filter->ysize; fj++)
				for (fi = 0; fi < filter->xsize; fi++)
					total += ((float) pr_get(tmp1, i + fi, j + fj) * filter->image[fj][fi]);
			pr_put(pr2, i, j, truncate((int)rint((total * scale) + offset), levels, 0));
		}
}


convolve_two_pass()
{
	float total, least, most;
	register int fj, fi;
	register int i, j;
	int     tmppix;
	Pixrect *tmp;
	if ((tmp = mem_create(pr1->pr_size.x, pr1->pr_size.y, 32)) == NULL)
		error("mem_create returned NULL");
	enlarge_image();
	/* first pass */
	least = (2147483647.0);
	most = (-2147483647.0);
	for (j = 0; j <  tmp1->pr_size.y - filter->ysize; j++)
		for (i = 0; i < tmp1->pr_size.x - filter->xsize; i++) {
			total = 0;
			for (fj = 0; fj < filter->ysize; fj++)
				for (fi = 0; fi < filter->xsize; fi++)
					total += ((float) pr_get(tmp1, i + fi, j + fj) * filter->image[fj][fi]);
			pr_put(tmp, i, j, (int)rint(total));
			if (total<least) least = total;
			most = MAX(total, most);
		}
	least = fabs(least);
	warning("least = %g, most = %g", least, most);
	/* calculate scale and offset */
	if (scale_method == SYMMETRIC) {
		scale = (float) levels / MAX(least, most);
		scale /= (filter->nflag) ? 2. : 1.;
		offset = (filter->nflag) ? levels / 2 : 0;
	} else {			    /* enhanced */
		scale = (float) levels / (least + most);
		offset = (filter->nflag) ? least * scale : 0;
	}

	warning("scale = %g on filter, output offset = %d on output", scale, offset);

	/* second pass */
	for (j = 0; j < pr1->pr_size.y; j++)
		for (i = 0; i < pr1->pr_size.x; i++) {
			tmppix = truncate((int) ((float) pr_get(tmp, i, j) * scale + offset), levels, 0);
			pr_put(pr2, i, j, tmppix);
		}
}


